/*

*/
import scene.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;
import java.awt.event.*;

/**
   This program uses Java's resize window event handler and
   the FrameBuffer's viewport to solve the problem of changing
   aspect ratios demonstrated by the program Circle_v1.java

   This program always uses a square viewport (which is the
   shape of the camera's view rectangle), which is as large as
   possible and is always positioned in the lower left hand corner
   of this program's FrameBuffer.

   As an exercise, modify this program to always place the
   square viewport in the upper left hand corner of the
   FrameBuffer.

   Also try placing the square viewport centered on the left
   edge of the FrameBuffer.

   Finally, try placing the square viewport at the center of
   the FrameBuffer.
*/
public class Circle_v2
{
   private static double left   = -1.0;
   private static double right  =  1.0;
   private static double bottom = -1.0;
   private static double top    =  1.0;

   public static void main(String[] args)
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the 'c' key to change the solid model colors.");
      System.out.println("Use the 'e' key to change the random edge colors.");
      System.out.println("Use the 'a' key to toggle antialiasing on and off.");

      // Create a Model object to hold the geometry.
      final Model model = new Model();

      // Add the geometry (in model coordinates) to the Model.
      int n = 50;
      Vertex[] v = new Vertex[n+1];

      // Create all the vertices around the rim of the circle.
      for (int j = 0; j < n; j++)
      {
         double c = Math.cos(j*(2.0*Math.PI)/n);
         double s = Math.sin(j*(2.0*Math.PI)/n);
         v[j] = new Vertex(c, s, 0);
      }
      v[n] = new Vertex(0, 0, 0);  // the center ofthe circle

      // Create line segments around the rim of the circle.
      LineSegment ls;
      for (int i = 0; i < n - 1; i++)
      {
         ls = new LineSegment( new Vertex(v[i]), new Vertex(v[i+1]) );
         model.addLineSegment( ls );
      }
      ls = new LineSegment( new Vertex(v[n-1]), new Vertex(v[0]) );
      model.addLineSegment( ls );
      // Create the spokes from the center of the circle.
      for (int i = 0; i < n; i++)
      {
         ls = new LineSegment( new Vertex(v[n]), new Vertex(v[i]) );
         model.addLineSegment( ls );
      }

      model.setColorRandom();

      // Push the model away from where the camera is.
      model.modelTranslate(0, 0, -1);

      // Create the Scene object that we shall render.
      final Scene scene = new Scene();

      // Add the Model to the Scene.
      scene.addModel(model);

      // Create a Camera object.
      Camera camera = new Camera();

      // Set up the camera's view volume.
      camera.projOrtho(left, right, bottom, top);

      // Add the Camera to the Scene.
      scene.setCamera(camera);


      // Create a InteractiveFrame and a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;

      @SuppressWarnings("serial")
      final InteractiveFrame app = new InteractiveFrame("Renderer 10", width, height)
      {
         // Implement part of the KeyListener interface.
         @Override public void keyTyped(KeyEvent e)
         {
            //System.out.println( e );
            char c = e.getKeyChar();
            if ('d' == c)
            {
               Pipeline.debug = ! Pipeline.debug;
             //Pipeline.debug_visible = ! Pipeline.debug_visible;
             //Rasterize.debug = ! Rasterize.debug;
             //RasterizeAntialias.debug = ! RasterizeAntialias.debug;
            }
            else if ('a' == c)
            {
               Pipeline.doAntialiasing = ! Pipeline.doAntialiasing;
            }
            else if ('c' == c)
            {
               // Change the solid random color of each model.
               for (Model m : scene.modelList)
                     m.setColorRandom();
            }
            else if ('e' == c)
            {
               // Change the random color at each edge.
               for (Model m : scene.modelList)
                  for (LineSegment ls : m.lineSegmentList)
                     ls.setColorRandom();
            }

            // Set up the camera's view volume.
            scene.camera.projOrtho(left, right, bottom, top);

            // Render again.
            this.getFrameBuffer().clearFB(Color.black);
            Pipeline.render(scene, this.getFrameBuffer());
            repaint();
         }

         // Implement part of the ComponentListener interface.
         @Override public void componentResized(ComponentEvent e)
         {
            System.out.println( e );
            java.awt.Container c = (java.awt.Container)e.getSource();
            java.awt.Insets insets = c.getInsets();
            int w = c.getWidth()  - insets.left - insets.right;
            int h = c.getHeight() - insets.top  - insets.bottom;

            this.createFrameBuffer(w, h);
            this.getFrameBuffer().clearFB(Color.black);
            int d = Math.min(w, h);
            this.getFrameBuffer().setViewport(0, h-d, d, d);
            Pipeline.render(scene, this.getFrameBuffer());
            repaint();
         }
      };

      app.addKeyListener(app);
      app.addComponentListener(app);
   }
}
