/*

*/
import scene.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;
import java.awt.event.*;

/**
  This program keeps the aspect ratio and the size of the complete
  circle at a constant 512 by 512 pixels. Also, the complete circle
  is anchored in the upper left hand corner of this program's window.
  But the program window can be any size. If the program window is
  smaller than 512 by 512 pixels, then only part of the circle is
  shown in the program window (the view of our scene gets "cropped"
  to fit inside of the screen window).

  Run this program and resize the window so that it is smaller than
  512 by 512 and see what happens.

  To create this effect, we call from within Java's componentResized()
  event handler the Camera's projOrtho() method. As the user changes
  the shape (the aspect ratio) and size of the screen window, we need
  to change the shape and size of the camera's view rectangle to always
  match the screen window.

  As an exercise, modify this program to always have the complete circle
  anchored in the lower right hand corner of this program's window. Try
  resizing the window using either the window's lower right hand corner
  or the window's upper left hand corner.

  As an exercise, modify this program to always have the complete circle
  centered at the center of the screen window (that is, centered in the
  FrameBuffer).
*/
public class Circle_v3
{
   private static double left   = -1.0;
   private static double right  =  1.0;
   private static double bottom = -1.0;
   private static double top    =  1.0;

   public static void main(String[] args)
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the 'c' key to change the solid model colors.");
      System.out.println("Use the 'e' key to change the random edge colors.");
      System.out.println("Use the 'a' key to toggle antialiasing on and off.");

      // Create a Model object to hold the geometry.
      final Model model = new Model();

      // Add the geometry (in model coordinates) to the Model.
      int n = 50;
      Vertex[] v = new Vertex[n+1];

      // Create all the vertices around the rim of the circle.
      for (int j = 0; j < n; j++)
      {
         double c = Math.cos(j*(2.0*Math.PI)/n);
         double s = Math.sin(j*(2.0*Math.PI)/n);
         v[j] = new Vertex(c, s, 0);
      }
      v[n] = new Vertex(0, 0, 0);  // the center ofthe circle

      // Create line segments around the rim of the circle.
      LineSegment ls;
      for (int i = 0; i < n - 1; i++)
      {
         ls = new LineSegment( new Vertex(v[i]), new Vertex(v[i+1]) );
         model.addLineSegment( ls );
      }
      ls = new LineSegment( new Vertex(v[n-1]), new Vertex(v[0]) );
      model.addLineSegment( ls );
      // Create the spokes from the center of the circle.
      for (int i = 0; i < n; i++)
      {
         ls = new LineSegment( new Vertex(v[n]), new Vertex(v[i]) );
         model.addLineSegment( ls );
      }

      model.setColorRandom();

      // Push the model away from where the camera is.
      model.modelTranslate(0, 0, -1);

      // Create the Scene object that we shall render.
      final Scene scene = new Scene();

      // Add the Model to the Scene.
      scene.addModel(model);

      // Create a Camera object.
      Camera camera = new Camera();

      // Set up the camera's view volume.
      camera.projOrtho(left, right, bottom, top);

      // Add the Camera to the Scene.
      scene.setCamera(camera);


      // Create a InteractiveFrame and a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;

      @SuppressWarnings("serial")
      final InteractiveFrame app = new InteractiveFrame("Renderer 10", width, height)
      {
         // Implement part of the KeyListener interface.
         @Override public void keyTyped(KeyEvent e)
         {
            //System.out.println( e );
            char c = e.getKeyChar();
            if ('d' == c)
            {
               Pipeline.debug = ! Pipeline.debug;
             //Pipeline.debug_visible = ! Pipeline.debug_visible;
             //Rasterize.debug = ! Rasterize.debug;
             //RasterizeAntialias.debug = ! RasterizeAntialias.debug;
            }
            else if ('a' == c)
            {
               Pipeline.doAntialiasing = ! Pipeline.doAntialiasing;
            }
            else if ('c' == c)
            {
               // Change the solid random color of each model.
               for (Model m : scene.modelList)
                     m.setColorRandom();
            }
            else if ('e' == c)
            {
               // Change the random color at each edge.
               for (Model m : scene.modelList)
                  for (LineSegment ls : m.lineSegmentList)
                     ls.setColorRandom();
            }

            // Set up the camera's view volume.
            scene.camera.projOrtho(left, right, bottom, top);

            // Render again.
            this.getFrameBuffer().clearFB(Color.black);
            Pipeline.render(scene, this.getFrameBuffer());
            repaint();
         }

         // Implement part of the ComponentListener interface.
         @Override public void componentResized(ComponentEvent e)
         {
            System.out.println( e );
            java.awt.Container c = (java.awt.Container)e.getSource();
            java.awt.Insets insets = c.getInsets();
            int w = c.getWidth()  - insets.left - insets.right;
            int h = c.getHeight() - insets.top  - insets.bottom;

            this.createFrameBuffer(w, h);
            this.getFrameBuffer().clearFB(Color.black);

            left = -1;
            right = -1 + (w/512.0)*2.0;
            bottom = 1 - (h/512.0)*2.0;
            top = 1;
            scene.camera.projOrtho(left, right, bottom, top);

            Pipeline.render(scene, this.getFrameBuffer());
            repaint();
         }
      };

      app.addKeyListener(app);
      app.addComponentListener(app);
   }
}
