/*

*/
import scene.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;
import java.awt.event.*;

/**
   This version adjusts the viewport within the FrameBuffer so that
   the aspect ratio of the graph is preserved and the shape of the
   graph is not distorted. But this results in a part of the screen
   window being not used for the graph, so in a sense, this wastes
   some screen real estate.
*/
public class SincFunction_v2
{
   private static double graphTop    =  1.0;
   private static double graphBottom = -0.22;
   private static double graphLeft   = -4.0;
   private static double graphRight  = - graphLeft;
   private static double graphWidth  = graphRight - graphLeft;
   private static double graphHeight = graphTop - graphBottom;
   private static double aspectRatio = graphWidth/graphHeight;

   public static void main(String[] args)
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the 'c' key to change the solid model colors.");
      System.out.println("Use the 'e' key to change the random edge colors.");
      System.out.println("Use the 'a' key to toggle antialiasing on and off.");

      // Create a Model object to hold the geometry.
      final Model model = new Model();

      // plot the sinc function using view coordinates
      LineSegment ls;
      double x = graphLeft;
      Vertex v0, v1;
      v0 = new Vertex(x, Math.sin(Math.PI*x)/(Math.PI*x), 0);
      x += 0.01;
      for(; x < graphRight; x += 0.01)  // draw the graph
      {
         v1 = new Vertex( x, Math.sin(Math.PI*x)/(Math.PI*x), 0);
         ls = new LineSegment( v0, v1 );
         model.addLineSegment( ls );
         v0 = new Vertex(v1);
      }

      model.setColorRandom();

      // Push the model away from where the camera is.
      model.modelTranslate(0, 0, -1);

      // Create the Scene object that we shall render.
      final Scene scene = new Scene();

      // Add the Model to the Scene.
      scene.addModel(model);

      // Create a Camera object.
      Camera camera = new Camera();

      // Set up the camera's view volume.
      camera.projOrtho(graphLeft, graphRight, graphBottom, graphTop);

      // Add the Camera to the Scene.
      scene.setCamera(camera);


      // Create a InteractiveFrame and a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;

      @SuppressWarnings("serial")
      final InteractiveFrame app = new InteractiveFrame("Renderer 10", width, height)
      {
         // Implement part of the KeyListener interface.
         @Override public void keyTyped(KeyEvent e)
         {
            //System.out.println( e );
            char c = e.getKeyChar();
            if ('d' == c)
            {
               Pipeline.debug = ! Pipeline.debug;
             //Pipeline.debug_visible = ! Pipeline.debug_visible;
             //Rasterize.debug = ! Rasterize.debug;
             //RasterizeAntialias.debug = ! RasterizeAntialias.debug;
            }
            else if ('a' == c)
            {
               Pipeline.doAntialiasing = ! Pipeline.doAntialiasing;
            }
            else if ('c' == c)
            {
               // Change the solid random color of each model.
               for (Model m : scene.modelList)
                     m.setColorRandom();
            }
            else if ('e' == c)
            {
               // Change the random color at each edge.
               for (Model m : scene.modelList)
                  for (LineSegment ls : m.lineSegmentList)
                     ls.setColorRandom();
            }

            // Set up the camera's view volume.
            camera.projOrtho(graphLeft, graphRight, graphBottom, graphTop);

            // Render again.
            this.getFrameBuffer().clearFB(Color.black);
            Pipeline.render(scene, this.getFrameBuffer());
            repaint();
         }

         // Implement part of the ComponentListener interface.
         @Override public void componentResized(ComponentEvent e)
         {
            System.out.println( e );
            java.awt.Container c = (java.awt.Container)e.getSource();
            java.awt.Insets insets = c.getInsets();
            int w = c.getWidth()  - insets.left - insets.right;
            int h = c.getHeight() - insets.top  - insets.bottom;

            this.createFrameBuffer(w, h);
            this.getFrameBuffer().clearFB(Color.black);

            // specify the viewport
            int viewportLeft, viewportBottom, viewportWidth, viewportHeight;
            if ( (double)w/(double)h >= aspectRatio )
            {
               viewportWidth  = (int)(h * aspectRatio);
               viewportHeight = h;
               viewportLeft   = (w - viewportWidth)/2;
               viewportBottom = 0;
            }
            else
            {
               viewportWidth = w;
               viewportHeight =  (int)(w / aspectRatio);
               viewportLeft = 0;
               viewportBottom = (h - viewportHeight)/2;
            }
            this.getFrameBuffer().setViewport(viewportLeft, viewportBottom,
                                              viewportWidth, viewportHeight);

            Pipeline.render(scene, this.getFrameBuffer());
            repaint();
         }
      };

      app.addKeyListener(app);
      app.addComponentListener(app);
   }
}
