/*

*/
import scene.*;
import pipeline.*;
import framebuffer.*;

import java.awt.Color;
import java.awt.event.*;

/**
   This version adjusts the viewport and the world window so that the
   aspect ratio of the graph is preserved (and the shape of the graph
   is not distorted) and also so that the entire height of the screen
   window is always used. This means that when the screen window is too
   narrow, part of the left and right ends of the graph must be cut off.
   Notice that "cutting off" the left and right ends of the graph is done
   by adjusting the camera's view volume..
*/
public class SincFunction_v3
{
   private static double graphTop    =  1.0;
   private static double graphBottom = -0.22;
   private static double graphLeft   = -4.0;
   private static double graphRight  = - graphLeft;
   private static double graphWidth  = graphRight - graphLeft;
   private static double graphHeight = graphTop - graphBottom;
   private static double aspectRatio = graphWidth/graphHeight;

   public static void main(String[] args)
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the 'c' key to change the solid model colors.");
      System.out.println("Use the 'e' key to change the random edge colors.");
      System.out.println("Use the 'a' key to toggle antialiasing on and off.");

      // Create a Model object to hold the geometry.
      final Model model = new Model();

      // plot the sinc function using view coordinates
      LineSegment ls;
      double x = graphLeft;
      Vertex v0, v1;
      v0 = new Vertex(x, Math.sin(Math.PI*x)/(Math.PI*x), 0);
      x += 0.01;
      for(; x < graphRight; x += 0.01)  // draw the graph
      {
         v1 = new Vertex( x, Math.sin(Math.PI*x)/(Math.PI*x), 0);
         ls = new LineSegment( v0, v1 );
         model.addLineSegment( ls );
         v0 = new Vertex(v1);
      }

      model.setColorRandom();

      // Push the model away from where the camera is.
      model.modelTranslate(0, 0, -1);

      // Create the Scene object that we shall render.
      final Scene scene = new Scene();

      // Add the Model to the Scene.
      scene.addModel(model);

      // Create a Camera object.
      Camera camera = new Camera();

      // Set up the camera's view volume.
      camera.projOrtho(graphLeft, graphRight, graphBottom, graphTop);

      // Add the Camera to the Scene.
      scene.setCamera(camera);


      // Create a InteractiveFrame and a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;

      @SuppressWarnings("serial")
      final InteractiveFrame app = new InteractiveFrame("Renderer 10", width, height)
      {
         // Implement part of the KeyListener interface.
         @Override public void keyTyped(KeyEvent e)
         {
            //System.out.println( e );
            char c = e.getKeyChar();
            if ('d' == c)
            {
               Pipeline.debug = ! Pipeline.debug;
             //Pipeline.debug_visible = ! Pipeline.debug_visible;
             //Rasterize.debug = ! Rasterize.debug;
             //RasterizeAntialias.debug = ! RasterizeAntialias.debug;
            }
            else if ('a' == c)
            {
               Pipeline.doAntialiasing = ! Pipeline.doAntialiasing;
            }
            else if ('c' == c)
            {
               // Change the solid random color of each model.
               for (Model m : scene.modelList)
                     m.setColorRandom();
            }
            else if ('e' == c)
            {
               // Change the random color at each edge.
               for (Model m : scene.modelList)
                  for (LineSegment ls : m.lineSegmentList)
                     ls.setColorRandom();
            }

            // Set up the camera's view volume.
            camera.projOrtho(graphLeft, graphRight, graphBottom, graphTop);

            // Render again.
            this.getFrameBuffer().clearFB(Color.black);
            Pipeline.render(scene, this.getFrameBuffer());
            repaint();
         }

         // Implement part of the ComponentListener interface.
         @Override public void componentResized(ComponentEvent e)
         {
            System.out.println( e );
            java.awt.Container c = (java.awt.Container)e.getSource();
            java.awt.Insets insets = c.getInsets();
            int w = c.getWidth()  - insets.left - insets.right;
            int h = c.getHeight() - insets.top  - insets.bottom;

            this.createFrameBuffer(w, h);
            this.getFrameBuffer().clearFB(Color.black);

            // specify the viewport
            int viewportLeft, viewportBottom, viewportWidth, viewportHeight;
            double worldLeftEdge, worldRightEdge;
            if ( (double)w/(double)h >= aspectRatio )
            {
               viewportWidth = (int)(w * aspectRatio);
               viewportLeft = (w - viewportWidth)/2;
            }
            else
            {
               viewportWidth = w;
               viewportLeft  = 0;
            }
            viewportHeight = h;  // always use the entire height of the screen window
            viewportBottom = 0;
            this.getFrameBuffer().setViewport(viewportLeft, viewportBottom,
                                              viewportWidth, viewportHeight);

            // specify the view volume
            if ( (double)w/(double)h >= aspectRatio )
            {
               worldLeftEdge = graphLeft;
            }
            else
            {
               if (viewportHeight != 0)  // check for minimized screen window
               {
                  worldLeftEdge =  -(((float)viewportWidth/(float)viewportHeight)*graphHeight)/2.0;
               }
               else
               {
                  worldLeftEdge = 0;
               }
            }
            System.out.printf("worldLeftEdge = %f, viewportHeight = %d\n", worldLeftEdge, viewportHeight);
            worldRightEdge = - worldLeftEdge;
            scene.camera.projOrtho(worldLeftEdge, worldRightEdge, graphBottom, graphTop);

            Pipeline.render(scene, this.getFrameBuffer());
            repaint();
         }
      };

      app.addKeyListener(app);
      app.addComponentListener(app);
   }
}
