/*

*/
package scene;

/**
   This Camera data structure represents a camera located
   at the origin and looking down the negative z-axis.

   This camera has associated to it a "view volume" that
   determines what part of space the camera "sees" in front
   of it when we use the camera to take a picture (that is,
   when we render a scene).

   This camera can "take a picture" two ways, using a
   perspective projection or using a parallel (orthographic)
   projection. Each way of taking a picture has a different
   shape for its view volume. The data in this data structure
   determines each of the two view volume shapes.

   For the perspective projection, the view volume (in view
   coordinates!) is an infinitely long pyramid that is formed
   by the pyramid with its apex at the origin and its base in
   the plane z = -near with edges x = left, x = right, y = top,
   and y = bottom.

   For the orthographic projection, the view volume (in view
   coordinates!) is an infinitely long rectangular cylinder
   parallel to the z-axis and with sides x = left, x = right,
   y = top, and y = bottom (an infinite parallelepiped).

   The part of space determined by the view volume is what
   the camera "sees" when we "take a picture" (that is,
   when we render a scene).

   The plane z = -near (in view coordinates) is the camera's view plane.
   The rectangle in the view plane with corners (left, bottom, -near)
   and (right, top, -near) is the camera's view rectangle. The view
   rectangle is like the film in a real camera, it is where the camera's
   image appears when you take a picture. The contents of the camera's
   view rectangle (after it gets "normalized") is what gets rasterized
   into the framebuffer.

   The projection matrix in this camera preserves some depth
   information. That is, this "projection" does not squash the
   camera's solid three-dimensional view volume into a flat
   two-dimensional rectangle.

   In fact, the perspective projection matrix in this class is a
   "homogeneous projection" that sends the camera's three-dimensional
   perspective view volume into a four-dimensional view region in
   four-dimensional homogeneous space. The perspective division
   operation will send that four-dimensional view region back to a
   three-dimensional view volume (in normalized device coordinates).

   The orthographic projection matrix in this class preserves depth
   information by simply being a translation matrix, so the solid
   three-dimensional orthographic view volume is just translated, and
   the perspective division operation has no affect on the view volume.
*/
public class Camera
{
   public boolean perspective;  // choose either perspective or parallel projection

   public Matrix normalizeMatrix;
   public Matrix projMatrix;

   public double left;   // These numbers define the camera's view volume.
   public double right;  // These numbers are encoded into the normalization matrix
   public double bottom;
   public double top;
   public double n;


   /**
      The default camera has the standard perspective view volume.
   */
   public Camera()
   {
      this.projPerspective(-1.0, 1.0, -1.0, 1.0, 1.0);
   }


   /**
      Set up this camera's view volume as a perspective projection
      of an infinite view pyramid.

      @param left    left edge of view rectangle in the plane z = -near
      @param right   right edge of view rectangle in the plane z = -near
      @param bottom  bottom edge of view rectangle in the plane z = -near
      @param top     top edge of view rectangle in the plane z = -near
      @param near    distance from the origin to the view plane
   */
   public void projPerspective(double left, double right, double bottom, double top, double near)
   {
      this.left   = left;    // These numbers define the camera's view volume.
      this.right  = right;   // These numbers are encoded into the normalization matrix
      this.bottom = bottom;
      this.top    = top;
      this.n      = -near;

      this.projMatrix = new PerspectiveProjectionMatrix1();
      this.normalizeMatrix = new PerspectiveNormalizeMatrix(left, right, bottom, top, near);

      this.perspective = true;
   }


   /**
      An alternative way to determine the camera's perspective view volume.

      Here, the view volume is determined by a vertical "field of view"
      angle and an aspect ratio for the view rectangle.

      @param fovy    angle in the y-direction subtended by the view rectangle
      @param aspect  aspect ratio of the view rectangle
      @param near    distance from the origin to the view plane
   */
   public void projPerspective(double fovy, double aspect, double near)
   {
      this.top    =  near * Math.tan((Math.PI/180.0)*fovy/2.0);
      this.bottom = -top;
      this.right  =  top * aspect;
      this.left   = -right;
      this.n      = -near;

      projPerspective(left, right, bottom, top, near);
   }


   /**
      Set up this camera's view volume as a parallel (orthographic) projection
      of an infinite view parallelepiped.

      @param left    left edge of view rectangle in the plane z = -near
      @param right   right edge of view rectangle in the plane z = -near
      @param bottom  bottom edge of view rectangle in the plane z = -near
      @param top     top edge of view rectangle in the plane z = -near
   */
   public void projOrtho(double left, double right, double bottom, double top)
   {
      this.left   = left;    // These numbers define the camera's view volume.
      this.right  = right;   // These numbers are encoded into the normalization matrix
      this.bottom = bottom;
      this.top    = top;
      this.n      = 0;

      this.projMatrix = new OrthographicProjectionMatrix1();
      this.normalizeMatrix = new OrthographicNormalizeMatrix(left, right, bottom, top);

      this.perspective = false;
   }


   /**
      For debugging.

      @return String representation of this Camera object
   */
   public String toString()
   {
      double fovy = 2.0 * (180./Math.PI) * Math.atan(top/(-n));
      String result = "";
      result += "Camera: \n";
      result += "perspective = " + perspective + "\n";
      result += "left = "   + left + ", "
             +  "right = "  + right + "\n"
             +  "bottom = " + bottom + ", "
             +  "top = "    + top + "\n"
             +  "near = "   + -n + "\n"
             +  "(fovy = " + fovy + ", aspect = " + right/top + ")\n"
             +  projMatrix;
      return result;
   }
}
